@@ -1,20 +1,8 @@
 /*
- *  Copyright 2016 Friedrich W. H. Kossebau  <kossebau@kde.org>
- *
- *  This library is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU Library General Public
- *  License version 2 as published by the Free Software Foundation;
- *
- *  This library is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- *  Library General Public License for more details.
- *
- *  You should have received a copy of the GNU Library General Public License
- *  along with this library; see the file COPYING.LIB.  If not, write to
- *  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- *  Boston, MA 02110-1301, USA.
- */
+    SPDX-FileCopyrightText: 2016 Friedrich W. H. Kossebau <kossebau@kde.org>
+
+    SPDX-License-Identifier: LGPL-2.0-only
+*/
 
 // test object
 #include <kaboutdata.h>
@@ -32,6 +20,9 @@ class KAboutDataApplicationDataTest : public QObject
 
 private Q_SLOTS:
     void testInteractionWithQApplicationData();
+#if KCOREADDONS_BUILD_DEPRECATED_SINCE(5, 76)
+    void testRegisterPluginData();
+#endif
 };
 
 
@@ -91,6 +82,26 @@ void KAboutDataApplicationDataTest::testInteractionWithQApplicationData()
     QCOMPARE(applicationAboutData2.desktopFileName(), QLatin1String(DesktopFileName2));
 }
 
+#if KCOREADDONS_BUILD_DEPRECATED_SINCE(5, 76)
+void KAboutDataApplicationDataTest::testRegisterPluginData()
+{
+    for (const auto &name : {QStringLiteral("foo"), QStringLiteral("bar")}) {
+        QVERIFY(!KAboutData::pluginData(name));
+        KAboutData::registerPluginData(KAboutData(name));
+
+        auto v1 = KAboutData::pluginData(name);
+        QVERIFY(v1);
+        QCOMPARE(v1->componentName(), name);
+
+        // re-registering will overwrite and not trigger memory leaks (check LSAN)
+        KAboutData::registerPluginData(KAboutData(name));
+
+        // the pointer staid the same, as QHash is node based
+        QCOMPARE(KAboutData::pluginData(name), v1);
+    }
+}
+#endif
+
 QTEST_MAIN(KAboutDataApplicationDataTest)
 
 #include "kaboutdataapplicationdatatest.moc"
