@@ -1,21 +1,9 @@
-/* This file is part of the KDE libraries
+/*
+    This file is part of the KDE libraries
 
-    Copyright (c) 2009 David Faure <faure@kde.org>
+    SPDX-FileCopyrightText: 2009 David Faure <faure@kde.org>
 
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Library General Public
-    License as published by the Free Software Foundation; either
-    version 2 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
-
-    You should have received a copy of the GNU Library General Public License
-    along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-    Boston, MA 02110-1301, USA.
+    SPDX-License-Identifier: LGPL-2.0-or-later
 */
 
 #include <kdirwatch.h>
@@ -23,9 +11,10 @@
 #include <QDir>
 #include <QDebug>
 #include <QFileInfo>
-#include <qtemporarydir.h>
+#include <QTemporaryDir>
 #include <QTest>
 #include <QSignalSpy>
+#include <QThread>
 #include <sys/stat.h>
 #ifdef Q_OS_UNIX
 #include <unistd.h> // ::link()
@@ -87,7 +76,8 @@ public:
 
         m_path = m_tempDir.path() + QLatin1Char('/');
         KDirWatch *dirW = &s_staticObject()->m_dirWatch;
-        m_slow = (dirW->internalMethod() == KDirWatch::FAM || dirW->internalMethod() == KDirWatch::Stat);
+        m_stat = dirW->internalMethod() == KDirWatch::Stat;
+        m_slow = (dirW->internalMethod() == KDirWatch::FAM || m_stat);
         qDebug() << "Using method" << methodToString(dirW->internalMethod());
     }
 
@@ -118,10 +108,10 @@ private Q_SLOTS: // test methods
     void nestedEventLoop();
     void testHardlinkChange();
     void stopAndRestart();
-    void shouldIgnoreQrcPaths();
     void benchCreateTree();
     void benchCreateWatcher();
     void benchNotifyWatcher();
+    void testRefcounting();
 
 protected Q_SLOTS: // internal slots
     void nestedEventLoopSlot();
@@ -145,6 +135,7 @@ private:
     QTemporaryDir m_tempDir;
     QString m_path;
     bool m_slow;
+    bool m_stat;
 };
 
 QTEST_MAIN(KDirWatch_UnitTest)
@@ -159,6 +150,14 @@ void KDirWatch_UnitTest::createFile(const QString &path)
 {
     QFile file(path);
     QVERIFY(file.open(QIODevice::WriteOnly));
+#ifdef Q_OS_FREEBSD
+    // FreeBSD has inotify implemented as user-space library over native kevent API.
+    // When using it, one has to open() a file to start watching it, so workaround
+    // test breakage by giving inotify time to react to file creation.
+    // Full context: https://github.com/libinotify-kqueue/libinotify-kqueue/issues/10
+    if(!m_slow)
+        QThread::msleep(1);
+#endif
     file.write(QByteArray("foo"));
     file.close();
     //qDebug() << path;
@@ -287,11 +286,11 @@ QList<QVariantList> KDirWatch_UnitTest::waitForDirtySignal(KDirWatch &watch, int
     while (spyDirty.count() < expected) {
         if (++numTries > s_maxTries) {
             qWarning() << "Timeout waiting for KDirWatch. Got" << spyDirty.count() << "dirty() signals, expected" << expected;
-            return spyDirty;
+            return std::move(spyDirty);
         }
         spyDirty.wait(50);
     }
-    return spyDirty;
+    return std::move(spyDirty);
 }
 
 bool KDirWatch_UnitTest::waitForOneSignal(KDirWatch &watch, const char *sig, const QString &path)
@@ -334,14 +333,28 @@ bool KDirWatch_UnitTest::waitForRecreationSignal(KDirWatch &watch, const QString
   // When watching for a deleted + created signal pair, the two might come so close that
   // using waitForOneSignal will miss the created signal.  This function monitors both all
   // the time to ensure both are received.
+  //
+  // In addition, it allows dirty() to be emitted (for that same path) as an alternative
 
   const QString expectedPath = removeTrailingSlash(path);
-  QSignalSpy spyDeleted(&watch, SIGNAL(deleted(QString)));
-  QSignalSpy spyCreated(&watch, SIGNAL(created(QString)));
+  QSignalSpy spyDirty(&watch, &KDirWatch::dirty);
+  QSignalSpy spyDeleted(&watch, &KDirWatch::deleted);
+  QSignalSpy spyCreated(&watch, &KDirWatch::created);
 
-  if(!spyDeleted.wait(50 * s_maxTries)) {
-    qWarning() << "Timeout waiting for KDirWatch signal deleted(QString) (" << path << ")";
-    return false;
+  int numTries = 0;
+  while (spyDeleted.isEmpty() && spyDirty.isEmpty()) {
+      if (++numTries > s_maxTries) {
+          return false;
+      }
+      spyDeleted.wait(50);
+      while (!spyDirty.isEmpty()) {
+          if (spyDirty.at(0).at(0).toString() != expectedPath) { // unrelated
+              spyDirty.removeFirst();
+          }
+      }
+  }
+  if (!spyDirty.isEmpty()) {
+      return true;
   }
 
   // Don't bother waiting for the created signal if the signal spy already received a signal.
@@ -361,11 +374,11 @@ QList<QVariantList> KDirWatch_UnitTest::waitForDeletedSignal(KDirWatch &watch, i
     while (spyDeleted.count() < expected) {
         if (++numTries > s_maxTries) {
             qWarning() << "Timeout waiting for KDirWatch. Got" << spyDeleted.count() << "deleted() signals, expected" << expected;
-            return spyDeleted;
+            return std::move(spyDeleted);
         }
         spyDeleted.wait(50);
     }
-    return spyDeleted;
+    return std::move(spyDeleted);
 }
 
 void KDirWatch_UnitTest::touchOneFile() // watch a dir, create a file in it
@@ -378,7 +391,7 @@ void KDirWatch_UnitTest::touchOneFile() // watch a dir, create a file in it
 
     // dirty(the directory) should be emitted.
     QSignalSpy spyCreated(&watch, SIGNAL(created(QString)));
-    const QString file0 = createFile(0);
+    createFile(0);
     QVERIFY(waitForOneSignal(watch, SIGNAL(dirty(QString)), m_path));
     QCOMPARE(spyCreated.count(), 0); // "This is not emitted when creating a file is created in a watched directory."
 
@@ -429,11 +442,13 @@ void KDirWatch_UnitTest::removeAndReAdd()
 {
     KDirWatch watch;
     watch.addDir(m_path);
+    // This triggers bug #374075.
+    watch.addDir(QStringLiteral(":/kio5/newfile-templates"));
     watch.startScan();
     if (watch.internalMethod() != KDirWatch::INotify) {
         waitUntilNewSecond();    // necessary for mtime checks in scanEntry
     }
-    const QString file0 = createFile(0);
+    createFile(0);
     QVERIFY(waitForOneSignal(watch, SIGNAL(dirty(QString)), m_path));
 
     // Just like KDirLister does: remove the watch, then re-add it.
@@ -442,8 +457,7 @@ void KDirWatch_UnitTest::removeAndReAdd()
     if (watch.internalMethod() != KDirWatch::INotify) {
         waitUntilMTimeChange(m_path);    // necessary for FAM and QFSWatcher
     }
-    const QString file1 = createFile(1);
-    //qDebug() << "created" << file1;
+    createFile(1);
     QVERIFY(waitForOneSignal(watch, SIGNAL(dirty(QString)), m_path));
 }
 
@@ -548,23 +562,59 @@ void KDirWatch_UnitTest::testDeleteAndRecreateFile() // Useful for /etc/localtim
     // Make sure this even works multiple times, as needed for ksycoca
     for (int i = 0; i < 5; ++i) {
 
+        if (m_slow || watch.internalMethod() == KDirWatch::QFSWatch) {
+            waitUntilNewSecond();
+        }
+
+        qDebug() << "Attempt #" << (i+1) << "removing+recreating" << file1;
+
+        // When watching for a deleted + created signal pair, the two might come so close that
+        // using waitForOneSignal will miss the created signal.  This function monitors both all
+        // the time to ensure both are received.
+        //
+        // In addition, allow dirty() to be emitted (for that same path) as an alternative
+
+        const QString expectedPath = file1;
+        QSignalSpy spyDirty(&watch, &KDirWatch::dirty);
+        QSignalSpy spyDeleted(&watch, &KDirWatch::deleted);
+        QSignalSpy spyCreated(&watch, &KDirWatch::created);
+
+        // WHEN
         QFile::remove(file1);
         // And recreate immediately, to try and fool KDirWatch with unchanged ctime/mtime ;)
         // (This emulates the /etc/localtime case)
         createFile(file1);
-
         //QCOMPARE(KDE::stat(QFile::encodeName(file1), &stat_buf), 0);
         //qDebug() << "new inode" << stat_buf.st_ino; // same!
 
-        {
-            QSignalSpy spyDirty(&watch, SIGNAL(dirty(QString)));
-            if(!waitForRecreationSignal(watch, file1)) {
-                // We may get a dirty signal here instead of a deleted/created set.
-                if (spyDirty.isEmpty() || !verifySignalPath(spyDirty, SIGNAL(dirty(QString)), file1)) {
-                    QFAIL("Failed to detect file deletion and recreation through either a deleted/created signal pair or through a dirty signal!");
+        // THEN
+        int numTries = 0;
+        while (spyDeleted.isEmpty() && spyDirty.isEmpty()) {
+            if (++numTries > s_maxTries) {
+                QFAIL("Failed to detect file deletion and recreation through either a deleted/created signal pair or through a dirty signal!");
+                return;
+            }
+            spyDeleted.wait(50);
+            while (!spyDirty.isEmpty()) {
+                if (spyDirty.at(0).at(0).toString() != expectedPath) { // unrelated
+                    spyDirty.removeFirst();
+                } else {
+                    break;
                 }
             }
         }
+        if (!spyDirty.isEmpty()) {
+            continue; // all ok
+        }
+
+        // Don't bother waiting for the created signal if the signal spy already received a signal.
+        if (spyCreated.isEmpty() && !spyCreated.wait(50 * s_maxTries)) {
+            qWarning() << "Timeout waiting for KDirWatch signal created(QString) (" << expectedPath << ")";
+            QFAIL("Timeout waiting for KDirWatch signal created, after deleted was emitted");
+            return;
+        }
+
+        QVERIFY(verifySignalPath(spyDeleted, "deleted(QString)", expectedPath) && verifySignalPath(spyCreated, "created(QString)", expectedPath));
     }
 
     waitUntilMTimeChange(file1);
@@ -786,29 +836,6 @@ void KDirWatch_UnitTest::stopAndRestart()
     QFile::remove(file3);
 }
 
-void KDirWatch_UnitTest::shouldIgnoreQrcPaths()
-{
-    const auto oldCwd = QDir::currentPath();
-    QVERIFY(QDir::setCurrent(QDir::homePath()));
-
-    KDirWatch watch;
-    watch.addDir(QDir::homePath());
-    // This triggers bug #374075.
-    watch.addDir(QStringLiteral(":/kio5/newfile-templates"));
-
-    QSignalSpy dirtySpy(&watch, &KDirWatch::dirty);
-
-    QFile file(QStringLiteral("bug374075.txt"));
-    QVERIFY(file.open(QIODevice::WriteOnly));
-    QVERIFY(file.write(QByteArrayLiteral("test")));
-    file.close();
-    QVERIFY(file.exists());
-    QVERIFY(dirtySpy.wait());
-    QVERIFY(dirtySpy.count() > 0);
-    QVERIFY(file.remove());
-    QVERIFY(QDir::setCurrent(oldCwd));
-}
-
 void KDirWatch_UnitTest::benchCreateTree()
 {
 #if !ENABLE_BENCHMARKS
@@ -857,4 +884,26 @@ void KDirWatch_UnitTest::benchNotifyWatcher()
     }
 }
 
+void KDirWatch_UnitTest::testRefcounting()
+{
+#if QT_CONFIG(cxx11_future)
+    bool initialExists = false;
+    bool secondExists = true; // the expectation is it will be set false
+    auto thread = QThread::create([&] {
+        QTemporaryDir dir;
+        {
+            KDirWatch watch;
+            watch.addFile(dir.path());
+            initialExists = KDirWatch::exists();
+        } // out of scope, the internal private should have been unset
+        secondExists = KDirWatch::exists();
+    });
+    thread->start();
+    thread->wait();
+    delete thread;
+    QVERIFY(initialExists);
+    QVERIFY(!secondExists);
+#endif
+}
+
 #include "kdirwatch_unittest.moc"
