@@ -1,20 +1,8 @@
 /*
-  Copyright (C) 2005 Ingo Kloecker <kloecker@kde.org>
-  Copyright (C) 2007 Allen Winter <winter@kde.org>
+    SPDX-FileCopyrightText: 2005 Ingo Kloecker <kloecker@kde.org>
+    SPDX-FileCopyrightText: 2007 Allen Winter <winter@kde.org>
 
-  This library is free software; you can redistribute it and/or
-  modify it under the terms of the GNU Library General Public
-  License version 2 as published by the Free Software Foundation.
-
-  This library is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  Library General Public License for more details.
-
-  You should have received a copy of the GNU Library General Public License
-  along with this library; see the file COPYING.LIB.  If not, write to
-  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-  Boston, MA 02110-1301, USA.
+    SPDX-License-Identifier: LGPL-2.0-only
 */
 
 #include "ktexttohtmltest.h"
@@ -118,7 +106,7 @@ void KTextToHTMLTest::testGetEmailAddress()
 void KTextToHTMLTest::testGetUrl()
 {
     QStringList brackets;
-    brackets << QStringLiteral("") << QStringLiteral("");   // no brackets
+    brackets << QString() << QString();   // no brackets
     brackets << QStringLiteral("<") << QStringLiteral(">");
     brackets << QStringLiteral("[") << QStringLiteral("]");
     brackets << QStringLiteral("\"") << QStringLiteral("\"");
@@ -160,7 +148,7 @@ void KTextToHTMLTest::testGetUrl2(const QString &left, const QString &right)
             QStringLiteral("\n\t  \n\t   a=1#anchor[bla]");
 
     for (const QString &schema : qAsConst(schemas)) {
-        for (QString url : qAsConst(urls)) {   //krazy:exclude=foreach
+        for (QString url : qAsConst(urls)) {
             // by definition: if the URL is enclosed in brackets, the URL itself is not allowed
             // to contain the closing bracket, as this would be detected as the end of the URL
             if ((left.length() == 1) && (url.contains(right[ 0 ]))) {
@@ -414,7 +402,7 @@ void KTextToHTMLTest::testHtmlConvert_data()
     QTest::newRow("url-in-parenthesis-3") << "bla (https://www.kde.org - section 5.2)"
                                           << KTextToHTML::Options(KTextToHTML::PreserveSpaces)
                                           << "bla (<a href=\"https://www.kde.org\">https://www.kde.org</a> - section 5.2)";
-    
+
     // Fix url as foo <<url> <url>> when we concatened them.
     QTest::newRow("url-with-url") << "foo <https://www.kde.org/ <https://www.kde.org/>>"
                                   << KTextToHTML::Options(KTextToHTML::PreserveSpaces)
@@ -529,6 +517,33 @@ void KTextToHTMLTest::testHtmlConvert_data()
     QTest::newRow("bug-414360") << "https://www.openstreetmap.org/directions?engine=graphhopper_foot&route=44.85765%2C-0.55931%3B44.85713%2C-0.56117#map=18/44.85756/-0.56094"
                      << KTextToHTML::Options(KTextToHTML::PreserveSpaces)
                      << "<a href=\"https://www.openstreetmap.org/directions?engine=graphhopper_foot&route=44.85765%2C-0.55931%3B44.85713%2C-0.56117#map=18/44.85756/-0.56094\">https://www.openstreetmap.org/directions?engine=graphhopper_foot&amp;route=44.85765%2C-0.55931%3B44.85713%2C-0.56117#map=18/44.85756/-0.56094</a>";
+
+    //xmpp bug 422291
+    QTest::newRow("xmpp1") << "xmpp:username@server.tld"
+                                       << KTextToHTML::Options(KTextToHTML::PreserveSpaces)
+                                       << "<a href=\"xmpp:username@server.tld\">xmpp:username@server.tld</a>";
+    QTest::newRow("xmpp2") << "xmpp:conversations@conference.siacs.eu"
+                                       << KTextToHTML::Options(KTextToHTML::PreserveSpaces)
+                                       << "<a href=\"xmpp:conversations@conference.siacs.eu\">xmpp:conversations@conference.siacs.eu</a>";
+    QTest::newRow("xmpp3") << "xmpp:conversations@conference.siacs.eu?join"
+                                       << KTextToHTML::Options(KTextToHTML::PreserveSpaces)
+                                       << "<a href=\"xmpp:conversations@conference.siacs.eu?join\">xmpp:conversations@conference.siacs.eu?join</a>";
+
+    //Test news: only
+    QTest::newRow("news") << "news: "
+                                       << KTextToHTML::Options(KTextToHTML::PreserveSpaces)
+                                       << "news:&nbsp;";
+
+    QTest::newRow("ftp") << "ftp: "
+                                       << KTextToHTML::Options(KTextToHTML::PreserveSpaces)
+                                       << "ftp:&nbsp;";
+    QTest::newRow("mailto") << "mailto: "
+                                       << KTextToHTML::Options(KTextToHTML::PreserveSpaces)
+                                       << "mailto:&nbsp;";
+    QTest::newRow("empty") << ""
+                                       << KTextToHTML::Options(KTextToHTML::PreserveSpaces)
+                                       << "";
+
 }
 
 
@@ -550,7 +565,7 @@ void KTextToHTMLTest::benchHtmlConvert_data()
     QTest::addColumn<QString>("text");
     QTest::addColumn<KTextToHTML::Options>("options");
 
-    auto text = QString(QStringLiteral("foo bar asdf :)")).repeated(1000);
+    auto text = QStringLiteral("foo bar asdf :)").repeated(1000);
     QTest::newRow("plain") << text << KTextToHTML::Options();
     QTest::newRow("preserve-spaces") << text << KTextToHTML::Options(KTextToHTML::PreserveSpaces);
     QTest::newRow("highlight-text") << text << KTextToHTML::Options(KTextToHTML::HighlightText);
