@@ -1,42 +1,29 @@
 /*
- * This file is part of the KDE project.
- * Copyright © 2010, 2012 Michael Pyne <mpyne@kde.org>
- * Copyright © 2012 Ralf Jung <ralfjung-e@gmx.de>
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
- * License version 2 as published by the Free Software Foundation.
- *
- * This library includes "MurmurHash" code from Austin Appleby, which is
- * placed in the public domain. See http://sites.google.com/site/murmurhash/
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
- *
- * You should have received a copy of the GNU Library General Public License
- * along with this library; see the file COPYING.LIB.  If not, write to
- * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301, USA.
- */
+    This file is part of the KDE project.
+
+    SPDX-FileCopyrightText: 2010, 2012 Michael Pyne <mpyne@kde.org>
+    SPDX-FileCopyrightText: 2012 Ralf Jung <ralfjung-e@gmx.de>
+
+    SPDX-License-Identifier: LGPL-2.0-only
+
+    This library includes "MurmurHash" code from Austin Appleby, which is
+    placed in the public domain. See http://sites.google.com/site/murmurhash/
+*/
 
 #include "kshareddatacache.h"
 #include "kshareddatacache_p.h" // Various auxiliary support code
 #include "kcoreaddons_debug.h"
 
-#include "qstandardpaths.h"
+#include <QStandardPaths>
 #include <qplatformdefs.h>
 
-#include <krandom.h>
-
-#include <QDebug>
 #include <QSharedPointer>
 #include <QByteArray>
 #include <QFile>
 #include <QAtomicInt>
 #include <QMutex>
 #include <QDir>
+#include <QRandomGenerator>
 
 #include <sys/types.h>
 #include <sys/mman.h>
@@ -59,7 +46,7 @@ class KSDCCorrupted
 public:
     KSDCCorrupted()
     {
-        qCritical() << "Error detected in cache, re-generating";
+        qCCritical(KCOREADDONS_DEBUG) << "Error detected in cache, re-generating";
     }
 };
 
@@ -89,7 +76,9 @@ static unsigned int MurmurHashAligned(const void *key, int len, unsigned int see
 
         switch (align) {
         case 1: t |= data[2] << 16;
+            Q_FALLTHROUGH();
         case 2: t |= data[1] << 8;
+            Q_FALLTHROUGH();
         case 3: t |= data[0];
         }
 
@@ -123,8 +112,11 @@ static unsigned int MurmurHashAligned(const void *key, int len, unsigned int see
 
         switch (pack) {
         case 3: d |= data[2] << 16;
+            Q_FALLTHROUGH();
         case 2: d |= data[1] << 8;
+            Q_FALLTHROUGH();
         case 1: d |= data[0];
+            Q_FALLTHROUGH();
         case 0: h += (t >> sr) | (d << sl);
             h *= m;
             h ^= h >> r;
@@ -148,7 +140,9 @@ static unsigned int MurmurHashAligned(const void *key, int len, unsigned int see
 
     switch (len) {
     case 3: h += data[2] << 16;
+        Q_FALLTHROUGH();
     case 2: h += data[1] << 8;
+        Q_FALLTHROUGH();
     case 1: h += data[0];
         h *= m;
         h ^= h >> r;
@@ -388,7 +382,7 @@ struct SharedMemory {
     // Returns pageSize in unsigned format.
     unsigned cachePageSize() const
     {
-        unsigned _pageSize = static_cast<unsigned>(pageSize.load());
+        unsigned _pageSize = static_cast<unsigned>(pageSize.loadRelaxed());
         // bits 9-18 may be set.
         static const unsigned validSizeMask = 0x7FE00u;
 
@@ -415,19 +409,19 @@ struct SharedMemory {
     bool performInitialSetup(uint _cacheSize, uint _pageSize)
     {
         if (_cacheSize < MINIMUM_CACHE_SIZE) {
-            qCritical() << "Internal error: Attempted to create a cache sized < "
+            qCCritical(KCOREADDONS_DEBUG) << "Internal error: Attempted to create a cache sized < "
                         << MINIMUM_CACHE_SIZE;
             return false;
         }
 
         if (_pageSize == 0) {
-            qCritical() << "Internal error: Attempted to create a cache with 0-sized pages.";
+            qCCritical(KCOREADDONS_DEBUG) << "Internal error: Attempted to create a cache with 0-sized pages.";
             return false;
         }
 
         shmLock.type = findBestSharedLock();
         if (shmLock.type == LOCKTYPE_INVALID) {
-            qCritical() << "Unable to find an appropriate lock to guard the shared cache. "
+            qCCritical(KCOREADDONS_DEBUG) << "Unable to find an appropriate lock to guard the shared cache. "
                         << "This *should* be essentially impossible. :(";
             return false;
         }
@@ -436,7 +430,7 @@ struct SharedMemory {
         QSharedPointer<KSDCLock> tempLock(createLockFromId(shmLock.type, shmLock));
 
         if (!tempLock->initialize(isProcessShared)) {
-            qCritical() << "Unable to initialize the lock for the cache!";
+            qCCritical(KCOREADDONS_DEBUG) << "Unable to initialize the lock for the cache!";
             return false;
         }
 
@@ -793,13 +787,13 @@ struct SharedMemory {
     uint removeUsedPages(uint numberNeeded)
     {
         if (numberNeeded == 0) {
-            qCritical() << "Internal error: Asked to remove exactly 0 pages for some reason.";
+            qCCritical(KCOREADDONS_DEBUG) << "Internal error: Asked to remove exactly 0 pages for some reason.";
             throw KSDCCorrupted();
         }
 
         if (numberNeeded > pageTableSize()) {
-            qCritical() << "Internal error: Requested more space than exists in the cache.";
-            qCritical() << numberNeeded << "requested, " << pageTableSize() << "is the total possible.";
+            qCCritical(KCOREADDONS_DEBUG) << "Internal error: Requested more space than exists in the cache.";
+            qCCritical(KCOREADDONS_DEBUG) << numberNeeded << "requested, " << pageTableSize() << "is the total possible.";
             throw KSDCCorrupted();
         }
 
@@ -819,7 +813,7 @@ struct SharedMemory {
             if (result < pageTableSize()) {
                 return result;
             } else {
-                qCritical() << "Just defragmented a locked cache, but still there"
+                qCCritical(KCOREADDONS_DEBUG) << "Just defragmented a locked cache, but still there"
                             << "isn't enough room for the current request.";
             }
         }
@@ -830,7 +824,7 @@ struct SharedMemory {
         QSharedPointer<IndexTableEntry> tablePtr(new IndexTableEntry[indexTableSize()], deleteTable);
 
         if (!tablePtr) {
-            qCritical() << "Unable to allocate temporary memory for sorting the cache!";
+            qCCritical(KCOREADDONS_DEBUG) << "Unable to allocate temporary memory for sorting the cache!";
             clearInternalTables();
             throw KSDCCorrupted();
         }
@@ -854,7 +848,7 @@ struct SharedMemory {
         // Declare the comparison function that we'll use to pass to qSort,
         // based on our cache eviction policy.
         bool (*compareFunction)(const IndexTableEntry &, const IndexTableEntry &);
-        switch (evictionPolicy.load()) {
+        switch (evictionPolicy.loadRelaxed()) {
         case KSharedDataCache::EvictLeastOftenUsed:
         case KSharedDataCache::NoEvictionPreference:
         default:
@@ -886,7 +880,7 @@ struct SharedMemory {
 
             // Removed everything, still no luck (or curIndex is set but too high).
             if (curIndex < 0 || static_cast<uint>(curIndex) >= indexTableSize()) {
-                qCritical() << "Trying to remove index" << curIndex
+                qCCritical(KCOREADDONS_DEBUG) << "Trying to remove index" << curIndex
                             << "out-of-bounds for index table of size" << indexTableSize();
                 throw KSDCCorrupted();
             }
@@ -996,7 +990,7 @@ public:
         m_lock.clear();
 
         if (shm && 0 != ::munmap(shm, m_mapSize)) {
-            qCritical() << "Unable to unmap shared memory segment"
+            qCCritical(KCOREADDONS_DEBUG) << "Unable to unmap shared memory segment"
                         << static_cast<void *>(shm) << ":" << ::strerror(errno);
         }
 
@@ -1026,7 +1020,7 @@ public:
         if (!QDir().mkpath(fileInfo.absolutePath())) {
             return;
         }
- 
+
         // The basic idea is to open the file that we want to map into shared
         // memory, and then actually establish the mapping. Once we have mapped the
         // file into shared memory we can close the file handle, the mapping will
@@ -1038,7 +1032,7 @@ public:
         void *mapAddress = MAP_FAILED;
 
         if (size < cacheSize) {
-            qCritical() << "Asked for a cache size less than requested size somehow -- Logic Error :(";
+            qCCritical(KCOREADDONS_DEBUG) << "Asked for a cache size less than requested size somehow -- Logic Error :(";
             return;
         }
 
@@ -1110,7 +1104,7 @@ public:
         // Well now we're really hosed. We can still work, but we can't even cache
         // data.
         if (mapAddress == MAP_FAILED) {
-            qCritical() << "Unable to allocate shared memory segment for shared data cache"
+            qCCritical(KCOREADDONS_DEBUG) << "Unable to allocate shared memory segment for shared data cache"
                         << cacheName << "of size" << cacheSize;
             return;
         }
@@ -1129,10 +1123,10 @@ public:
         //         1 means "in progress of initing"
         //         2 means "ready"
         uint usecSleepTime = 8; // Start by sleeping for 8 microseconds
-        while (shm->ready.load() != 2) {
+        while (shm->ready.loadRelaxed() != 2) {
             if (Q_UNLIKELY(usecSleepTime >= (1 << 21))) {
                 // Didn't acquire within ~8 seconds?  Assume an issue exists
-                qCritical() << "Unable to acquire shared lock, is the cache corrupt?";
+                qCCritical(KCOREADDONS_DEBUG) << "Unable to acquire shared lock, is the cache corrupt?";
 
                 file.remove(); // Unlink the cache in case it's corrupt.
                 detachFromSharedMemory();
@@ -1141,7 +1135,7 @@ public:
 
             if (shm->ready.testAndSetAcquire(0, 1)) {
                 if (!shm->performInitialSetup(cacheSize, pageSize)) {
-                    qCritical() << "Unable to perform initial setup, this system probably "
+                    qCCritical(KCOREADDONS_DEBUG) << "Unable to perform initial setup, this system probably "
                                 "does not really support process-shared pthreads or "
                                 "semaphores, even though it claims otherwise.";
 
@@ -1162,7 +1156,7 @@ public:
         bool isProcessSharingSupported = false;
 
         if (!m_lock->initialize(isProcessSharingSupported)) {
-            qCritical() << "Unable to setup shared cache lock, although it worked when created.";
+            qCCritical(KCOREADDONS_DEBUG) << "Unable to setup shared cache lock, although it worked when created.";
             detachFromSharedMemory();
         }
     }
@@ -1244,7 +1238,7 @@ public:
                 }
 
                 if (lockCount++ > 4) {
-                    qCritical() << "There is a very serious problem with the KDE data cache"
+                    qCCritical(KCOREADDONS_DEBUG) << "There is a very serious problem with the KDE data cache"
                                 << d->m_cacheName << "giving up trying to access cache.";
                     d->detachFromSharedMemory();
                     return false;
@@ -1268,7 +1262,7 @@ public:
             if (Q_UNLIKELY(d->shm->version != SharedMemory::PIXMAP_CACHE_VERSION)) {
                 return false;
             }
-            switch (d->shm->evictionPolicy.load()) {
+            switch (d->shm->evictionPolicy.loadRelaxed()) {
             case NoEvictionPreference:   // fallthrough
             case EvictLeastRecentlyUsed: // fallthrough
             case EvictLeastOftenUsed:    // fallthrough
@@ -1333,7 +1327,7 @@ void SharedMemory::removeEntry(uint index)
     }
 
     if (index != static_cast<uint>(pageTableEntries[firstPage].index)) {
-        qCritical() << "Removing entry" << index << "but the matching data"
+        qCCritical(KCOREADDONS_DEBUG) << "Removing entry" << index << "but the matching data"
                     << "doesn't link back -- cache is corrupt, clearing.";
         throw KSDCCorrupted();
     }
@@ -1347,7 +1341,7 @@ void SharedMemory::removeEntry(uint index)
     }
 
     if ((cacheAvail - savedCacheSize) != entriesToRemove) {
-        qCritical() << "We somehow did not remove" << entriesToRemove
+        qCCritical(KCOREADDONS_DEBUG) << "We somehow did not remove" << entriesToRemove
                     << "when removing entry" << index << ", instead we removed"
                     << (cacheAvail - savedCacheSize);
         throw KSDCCorrupted();
@@ -1389,7 +1383,7 @@ KSharedDataCache::KSharedDataCache(const QString &cacheName,
         try {
             d = new Private(cacheName, defaultCacheSize, expectedItemSize);
         } catch (KSDCCorrupted) {
-            qCritical()
+            qCCritical(KCOREADDONS_DEBUG)
                     << "Even a brand-new cache starts off corrupted, something is"
                     << "seriously wrong. :-(";
             d = nullptr; // Just in case
@@ -1451,7 +1445,7 @@ bool KSharedDataCache::insert(const QString &key, const QByteArray &data)
             cullCollisions = true;
         } else if (loadFactor > startCullPoint) {
             const int tripWireValue = RAND_MAX * (loadFactor - startCullPoint) / (mustCullPoint - startCullPoint);
-            if (KRandom::random() >= tripWireValue) {
+            if (QRandomGenerator::global()->bounded(RAND_MAX) >= tripWireValue) {
                 cullCollisions = true;
             }
         }
@@ -1529,7 +1523,7 @@ bool KSharedDataCache::insert(const QString &key, const QByteArray &data)
 
             if (firstPage >= d->shm->pageTableSize() ||
                     d->shm->cacheAvail < pagesNeeded) {
-                qCritical() << "Unable to free up memory for" << key;
+                qCCritical(KCOREADDONS_DEBUG) << "Unable to free up memory for" << key;
                 return false;
             }
         }
