@@ -4,6 +4,7 @@
    SPDX-FileCopyrightText: 2007 Flavio Castelli <flavio.castelli@gmail.com>
    SPDX-FileCopyrightText: 2008 Rafal Rzepecki <divided.mind@gmail.com>
    SPDX-FileCopyrightText: 2010 David Faure <faure@kde.org>
+   SPDX-FileCopyrightText: 2020 Harald Sitter <sitter@kde.org>
 
    SPDX-License-Identifier: LGPL-2.0-only
 */
@@ -87,6 +88,11 @@ static KDirWatchPrivate *createPrivate()
     }
     return dwp_self.localData();
 }
+static void destroyPrivate()
+{
+    dwp_self.localData()->deleteLater();
+    dwp_self.setLocalData(nullptr);
+}
 
 // Convert a string into a watch Method
 static KDirWatch::Method methodFromString(const QByteArray &method)
@@ -166,7 +172,8 @@ KDirWatchPrivate::KDirWatchPrivate()
 #if HAVE_SYS_INOTIFY_H
       mSn(nullptr),
 #endif
-      _isStopped(false)
+      _isStopped(false),
+      m_references(0)
 {
     // Debug unittest on CI
     if (qAppName() == QLatin1String("kservicetest") || qAppName() == QLatin1String("filetypestest")) {
@@ -668,14 +675,21 @@ bool KDirWatchPrivate::useFAM(Entry *e)
 
     // handle FAM events to avoid deadlock
     // (FAM sends back all files in a directory when monitoring)
-    do {
-      famEventReceived();
-      if (startedFAMMonitor && !e->m_famReportedSeen) {
-        // 50 is ~half the time it takes to setup a watch.  If gamin's latency
-        // gets better, this can be reduced.
-        QThread::msleep(50);
-      }
-    } while (startedFAMMonitor &&!e->m_famReportedSeen);
+    const int iterationCap = 80;
+    for (int i = 0; i <= iterationCap; ++i) { // we'll not wait forever; blocking for 4s seems plenty
+        famEventReceived();
+        // NB: check use_fam, if fam is defunct event receiving might disable fam support!
+        if (use_fam && startedFAMMonitor && !e->m_famReportedSeen) {
+            // 50 is ~half the time it takes to setup a watch.  If gamin's latency
+            // gets better, this can be reduced.
+            QThread::msleep(50);
+        } else if (use_fam && i == iterationCap) {
+            disableFAM();
+            return false;
+        } else {
+            break;
+        }
+    }
 
     return true;
 }
@@ -1577,6 +1591,19 @@ bool KDirWatchPrivate::isNoisyFile(const char *filename)
     return false;
 }
 
+void KDirWatchPrivate::ref()
+{
+    ++m_references;
+}
+
+void KDirWatchPrivate::unref()
+{
+    --m_references;
+    if (m_references == 0) {
+        destroyPrivate();
+    }
+}
+
 #if HAVE_FAM
 void KDirWatchPrivate::famEventReceived()
 {
@@ -1588,17 +1615,7 @@ void KDirWatchPrivate::famEventReceived()
 
     while (use_fam && FAMPending(&fc)) {
         if (FAMNextEvent(&fc, &fe) == -1) {
-            qCWarning(KCOREADDONS_DEBUG) << "FAM connection problem, switching to polling.";
-            use_fam = false;
-            delete sn; sn = nullptr;
-
-            // Replace all FAMMode entries with INotify/Stat
-            EntryMap::Iterator it = m_mapEntries.begin();
-            for (; it != m_mapEntries.end(); ++it)
-                if ((*it).m_mode == FAMMode && !(*it).m_clients.empty()) {
-                    Entry *e = &(*it);
-                    addWatch(e);
-                }
+            disableFAM();
         } else {
             checkFAMEvent(&fe);
         }
@@ -1607,6 +1624,22 @@ void KDirWatchPrivate::famEventReceived()
     QTimer::singleShot(0, this, SLOT(slotRemoveDelayed()));
 }
 
+void KDirWatchPrivate::disableFAM()
+{
+    qCWarning(KCOREADDONS_DEBUG) << "FAM connection problem, switching to a different system.";
+    use_fam = false;
+    delete sn;
+    sn = nullptr;
+
+    // Replace all FAMMode entries with another system (INotify/QFSW/Stat)
+    for (auto it = m_mapEntries.begin(); it != m_mapEntries.end(); ++it) {
+        if ((*it).m_mode == FAMMode && !(*it).m_clients.empty()) {
+            Entry *e = &(*it);
+            addWatch(e);
+        }
+    }
+}
+
 void KDirWatchPrivate::checkFAMEvent(FAMEvent *fe)
 {
     //qCDebug(KDIRWATCH);
@@ -1859,6 +1892,7 @@ static void postRoutine_KDirWatch()
 KDirWatch::KDirWatch(QObject *parent)
     : QObject(parent), d(createPrivate())
 {
+    d->ref();
     static QBasicAtomicInt nameCounter = Q_BASIC_ATOMIC_INITIALIZER(1);
     const int counter = nameCounter.fetchAndAddRelaxed(1); // returns the old value
     setObjectName(QStringLiteral("KDirWatch-%1").arg(counter));
@@ -1873,6 +1907,7 @@ KDirWatch::~KDirWatch()
 {
     if (d && dwp_self.hasLocalData()) { // skip this after app destruction
         d->removeEntries(this);
+        d->unref();
     }
 }
 
@@ -2016,22 +2051,28 @@ KDirWatch::Method KDirWatch::internalMethod() const
 {
     // This reproduces the logic in KDirWatchPrivate::addWatch
     switch (d->m_preferredMethod) {
+    case KDirWatch::FAM:
 #if HAVE_FAM
-    case KDirWatch::FAM: if (d->use_fam) {
+        if (d->use_fam) {
             return KDirWatch::FAM;
         }
-        break;
 #endif
+        break;
+    case KDirWatch::INotify:
 #if HAVE_SYS_INOTIFY_H
-    case KDirWatch::INotify: if (d->supports_inotify) {
+        if (d->supports_inotify) {
             return KDirWatch::INotify;
         }
+#endif
+        break;
+    case KDirWatch::QFSWatch:
+#if HAVE_QFILESYSTEMWATCHER
+        return KDirWatch::QFSWatch;
+#else
         break;
 #endif
-#if HAVE_QFILESYSTEMWATCHER
-    case KDirWatch::QFSWatch: return KDirWatch::QFSWatch;
-#endif
-    case KDirWatch::Stat: return KDirWatch::Stat;
+    case KDirWatch::Stat:
+        return KDirWatch::Stat;
     }
 
 #if HAVE_SYS_INOTIFY_H
