@@ -1,27 +1,16 @@
-/*  This file is part of the KDE libraries
-    Copyright (C) 2006 Michaël Larouche <michael.larouche@kdemail.net>
+/*
+    This file is part of the KDE libraries
 
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Library General Public
-    License as published by the Free Software Foundation; version 2
-    of the License.
+    SPDX-FileCopyrightText: 2006 Michaël Larouche <michael.larouche@kdemail.net>
 
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
-
-    You should have received a copy of the GNU Library General Public License
-    along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-    Boston, MA 02110-1301, USA.
+    SPDX-License-Identifier: LGPL-2.0-only
 */
 #ifndef KDECORE_KMESSAGE_H
 #define KDECORE_KMESSAGE_H
 
 #include <kcoreaddons_export.h>
 
-#include <QtCore/QString>
+#include <QString>
 
 class KMessageHandler;
 /**
