@@ -1,22 +1,9 @@
 /*
     This file is part of the KDE libraries
 
-    Copyright (C) 2007 Oswald Buddenhagen <ossi@kde.org>
+    SPDX-FileCopyrightText: 2007 Oswald Buddenhagen <ossi@kde.org>
 
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Library General Public
-    License as published by the Free Software Foundation; either
-    version 2 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
-
-    You should have received a copy of the GNU Library General Public License
-    along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-    Boston, MA 02110-1301, USA.
+    SPDX-License-Identifier: LGPL-2.0-or-later
 */
 
 #ifndef KPROCESS_H
@@ -304,6 +291,7 @@ public:
      */
     static int startDetached(const QStringList &argv);
 
+#if KCOREADDONS_ENABLE_DEPRECATED_SINCE(5, 78)
     /**
      * Obtain the process' ID as known to the system.
      *
@@ -313,8 +301,11 @@ public:
      * It cannot be applied to detached processes.
      *
      * @return the process ID
+     * @deprecated since 5.78, use processId()
      */
+    KCOREADDONS_DEPRECATED_VERSION(5, 78, "Use processId()")
     int pid() const;
+#endif
 
 protected:
     /**
