@@ -1,30 +1,19 @@
-/******************************************************************************
- *  Copyright 2013-2014 Sebastian Kügler <sebas@kde.org>                      *
- *  Copyright 2014 Alex Richardson <arichardson.kde@gmail.com>                *
- *                                                                            *
- *  This library is free software; you can redistribute it and/or             *
- *  modify it under the terms of the GNU Lesser General Public                *
- *                                                                            *
- *  License as published by the Free Software Foundation; either              *
- *  version 2.1 of the License, or (at your option) version 3, or any         *
- *  later version accepted by the membership of KDE e.V. (or its              *
- *  successor approved by the membership of KDE e.V.), which shall            *
- *  act as a proxy defined in Section 6 of version 3 of the license.          *
- *                                                                            *
- *  This library is distributed in the hope that it will be useful,           *
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU         *
- *  Lesser General Public License for more details.                           *
- *                                                                            *
- *  You should have received a copy of the GNU Lesser General Public          *
- *  License along with this library.  If not, see                             *
- *  <http://www.gnu.org/licenses/>.                                           *
- *                                                                            *
- ******************************************************************************/
+/*
+    SPDX-FileCopyrightText: 2013-2014 Sebastian Kügler <sebas@kde.org>
+    SPDX-FileCopyrightText: 2014 Alex Richardson <arichardson.kde@gmail.com>
+
+    SPDX-License-Identifier: LGPL-2.1-only OR LGPL-3.0-only OR LicenseRef-KDE-Accepted-LGPL
+*/
 
 
 #include "desktopfileparser_p.h"
 
+#ifdef BUILDING_DESKTOPTOJSON_TOOL
+#include "desktoptojson_debug.h"
+#else
+#include "desktopfileparser_debug.h"
+#endif
+// Qt
 #include <QCache>
 #include <QDir>
 #include <QFile>
@@ -35,14 +24,6 @@
 #include <QRegularExpression>
 #include <QRegularExpressionMatch>
 
-// in the desktoptojson binary enable debug messages by default, in the library only warning messages
-#ifdef BUILDING_DESKTOPTOJSON_TOOL
-Q_LOGGING_CATEGORY(DESKTOPPARSER, "kf5.kcoreaddons.desktopparser", QtDebugMsg)
-#else
-Q_LOGGING_CATEGORY(DESKTOPPARSER, "kf5.kcoreaddons.desktopparser", QtWarningMsg)
-#endif
-
-
 #ifdef BUILDING_DESKTOPTOJSON_TOOL
 // use if not else to prevent wrong scoping
 #define DESKTOPTOJSON_VERBOSE_DEBUG if (!DesktopFileParser::s_verbose) {} else qCDebug(DESKTOPPARSER)
@@ -195,7 +176,7 @@ bool readUntilDesktopEntryGroup(QFile &file, const QString &path, int &lineNr)
     }
     // we only convert data inside the [Desktop Entry] group
     while (!file.atEnd()) {
-        QByteArray line = file.readLine().trimmed();
+        const QByteArray line = file.readLine().trimmed();
         lineNr++;
         if (line == "[Desktop Entry]") {
             return true;
@@ -206,7 +187,7 @@ bool readUntilDesktopEntryGroup(QFile &file, const QString &path, int &lineNr)
 }
 
 
-QByteArray readTypeEntryForCurrentGroup(QFile &df, QByteArray *nextGroup)
+QByteArray readTypeEntryForCurrentGroup(QFile &df, QByteArray *nextGroup, QByteArray *pName)
 {
     QByteArray group = *nextGroup;
     QByteArray type;
@@ -234,6 +215,13 @@ QByteArray readTypeEntryForCurrentGroup(QFile &df, QByteArray *nextGroup)
         const auto match = typeEntryRegex.match(QString::fromUtf8(line));
         if (match.hasMatch()) {
             type = match.captured(1).toUtf8();
+        } else if (pName) {
+            const static QRegularExpression nameEntryRegex(
+                    QStringLiteral("^X-KDE-ServiceType\\s*=\\s*(.*)$"));
+            const auto nameMatch = nameEntryRegex.match(QString::fromUtf8(line));
+            if (nameMatch.hasMatch()) {
+                *pName = nameMatch.captured(1).toUtf8();
+            }
         }
     }
     return type;
@@ -290,7 +278,7 @@ static QString locateRelativeServiceType(const QString &relPath)
                                   QStringLiteral("kservicetypes5/") + relPath);
 }
 
-static QVector<CustomPropertyDefinition>* parseServiceTypesFile(const QString &inputPath)
+static ServiceTypeDefinition* parseServiceTypesFile(const QString &inputPath)
 {
     int lineNr = 0;
     QString path = inputPath;
@@ -298,7 +286,7 @@ static QVector<CustomPropertyDefinition>* parseServiceTypesFile(const QString &i
         path = locateRelativeServiceType(path);
         QString rcPath;
         if (path.isEmpty()) {
-            rcPath = QStringLiteral(":/kservicetypes5/") + inputPath;
+            rcPath = QLatin1String(":/kservicetypes5/") + inputPath;
             if (QFileInfo::exists(rcPath)) {
                 path = rcPath;
             }
@@ -316,12 +304,12 @@ static QVector<CustomPropertyDefinition>* parseServiceTypesFile(const QString &i
     if (!readUntilDesktopEntryGroup(df, path, lineNr)) {
         return nullptr;
     }
-    QVector<CustomPropertyDefinition> result;
+    ServiceTypeDefinition result;
     // TODO: passing nextGroup by pointer is inefficient as it will make deep copies every time
     // Not exactly performance critical code though so low priority
     QByteArray nextGroup = "Desktop Entry";
     // Type must be ServiceType now
-    QByteArray typeStr = readTypeEntryForCurrentGroup(df, &nextGroup);
+    QByteArray typeStr = readTypeEntryForCurrentGroup(df, &nextGroup, &result.m_serviceTypeName);
     if (typeStr != QByteArrayLiteral("ServiceType")) {
         qCWarning(DESKTOPPARSER) << path << "is not a valid service type: Type entry should be 'ServiceType', got"
             << typeStr << "instead.";
@@ -329,7 +317,7 @@ static QVector<CustomPropertyDefinition>* parseServiceTypesFile(const QString &i
     }
     while (!df.atEnd()) {
         QByteArray currentGroup = nextGroup;
-        typeStr = readTypeEntryForCurrentGroup(df, &nextGroup);
+        typeStr = readTypeEntryForCurrentGroup(df, &nextGroup, nullptr);
         if (!currentGroup.startsWith(QByteArrayLiteral("PropertyDef::"))) {
             qCWarning(DESKTOPPARSER) << "Skipping invalid group" << currentGroup << "in service type" << path;
             continue;
@@ -347,7 +335,7 @@ static QVector<CustomPropertyDefinition>* parseServiceTypesFile(const QString &i
             case QVariant::Double:
             case QVariant::Bool:
                 qCDebug(DESKTOPPARSER) << "Found property definition" << propertyName << "with type" << typeStr;
-                result.push_back(CustomPropertyDefinition(propertyName, type));
+                result.m_propertyDefs.push_back(CustomPropertyDefinition(propertyName, type));
                 break;
             case QVariant::Invalid:
                 qCWarning(DESKTOPPARSER) << "Property type" << typeStr << "is not a known QVariant type."
@@ -358,27 +346,23 @@ static QVector<CustomPropertyDefinition>* parseServiceTypesFile(const QString &i
                         << "found in" << path << "\nOnly QString, QStringList, int, double and bool are supported.";
         }
     }
-    return new QVector<CustomPropertyDefinition>(result);
+    return new ServiceTypeDefinition(result);
 }
 
 // a lazy map of service type definitions
-typedef QCache<QString, QVector<CustomPropertyDefinition>> ServiceTypesHash;
+typedef QCache<QString /*path*/, ServiceTypeDefinition> ServiceTypesHash;
 Q_GLOBAL_STATIC(ServiceTypesHash, s_serviceTypes)
 // access must be guarded by serviceTypesMutex as this code could be executed by multiple threads
 QBasicMutex s_serviceTypesMutex;
 } // end of anonymous namespace
 
 
-ServiceTypeDefinition::ServiceTypeDefinition()
+ServiceTypeDefinitions ServiceTypeDefinitions::fromFiles(const QStringList &paths)
 {
-}
-
-ServiceTypeDefinition ServiceTypeDefinition::fromFiles(const QStringList &paths)
-{
-    ServiceTypeDefinition ret;
+    ServiceTypeDefinitions ret;
     ret.m_definitions.reserve(paths.size());
     // as we might modify the cache we need to acquire a mutex here
-    foreach (const QString &serviceTypePath, paths) {
+    for (const QString &serviceTypePath : paths) {
         bool added = ret.addFile(serviceTypePath);
         if (!added) {
 #ifdef BUILDING_DESKTOPTOJSON_TOOL
@@ -389,16 +373,15 @@ ServiceTypeDefinition ServiceTypeDefinition::fromFiles(const QStringList &paths)
     return ret;
 }
 
-bool ServiceTypeDefinition::addFile(const QString& path)
+bool ServiceTypeDefinitions::addFile(const QString& path)
 {
     QMutexLocker lock(&s_serviceTypesMutex);
-    QVector<CustomPropertyDefinition>* def = s_serviceTypes->object(path);
+    ServiceTypeDefinition* def = s_serviceTypes->object(path);
 
     if (def) {
         // in cache but we still must make our own copy
         m_definitions << *def;
-    }
-    else {
+    } else {
         // not found in cache -> we need to parse the file
         qCDebug(DESKTOPPARSER) << "About to parse service type file" << path;
         def = parseServiceTypesFile(path);
@@ -412,19 +395,29 @@ bool ServiceTypeDefinition::addFile(const QString& path)
     return true;
 }
 
-QJsonValue ServiceTypeDefinition::parseValue(const QByteArray &key, const QString &value) const
+QJsonValue ServiceTypeDefinitions::parseValue(const QByteArray &key, const QString &value) const
 {
     // check whether the key has a special type associated with it
-    foreach (const CustomPropertyDefinition &propertyDef, m_definitions) {
-        if (propertyDef.key == key) {
-            return propertyDef.fromString(value);
+    for (const auto &def : m_definitions) {
+        for (const CustomPropertyDefinition &propertyDef : def.m_propertyDefs) {
+            if (propertyDef.key == key) {
+                return propertyDef.fromString(value);
+            }
         }
     }
     qCDebug(DESKTOPPARSER) << "Unknown property type for key" << key << "-> falling back to string";
     return QJsonValue(value);
 }
 
-void DesktopFileParser::convertToJson(const QByteArray &key, ServiceTypeDefinition &serviceTypes, const QString &value,
+bool ServiceTypeDefinitions::hasServiceType(const QByteArray &serviceTypeName) const
+{
+    const auto it = std::find_if(m_definitions.begin(), m_definitions.end(), [&serviceTypeName](const ServiceTypeDefinition &def) {
+            return def.m_serviceTypeName == serviceTypeName;
+    });
+    return it != m_definitions.end();
+}
+
+void DesktopFileParser::convertToJson(const QByteArray &key, ServiceTypeDefinitions &serviceTypes, const QString &value,
                                       QJsonObject &json, QJsonObject &kplugin, int lineNr)
 {
     /* The following keys are recognized (and added to a "KPlugin" object):
@@ -439,12 +432,16 @@ void DesktopFileParser::convertToJson(const QByteArray &key, ServiceTypeDefiniti
 
         X-KDE-PluginInfo-Author=Author's Name
         X-KDE-PluginInfo-Email=author@foo.bar
+        # alternatively to X-KDE-PluginInfo-Author & X-KDE-PluginInfo-Email:
+        X-KDE-PluginInfo-Authors=Author A's Name;AuthorB's Name (since KF 5.77)
+        X-KDE-PluginInfo-Emails=authorA@foo.bar;authorB@foo.bar (since KF 5.77)
         X-KDE-PluginInfo-Name=internalname
         X-KDE-PluginInfo-Version=1.1
         X-KDE-PluginInfo-Website=http://www.plugin.org/
         X-KDE-PluginInfo-Category=playlist
         X-KDE-PluginInfo-Depends=plugin1,plugin3
         X-KDE-PluginInfo-License=GPL
+        X-KDE-PluginInfo-Copyright=Copyright <year> by Author's Name (since KF 5.77, translatable)
         X-KDE-PluginInfo-EnabledByDefault=true
         X-KDE-FormFactors=desktop
     */
@@ -456,6 +453,11 @@ void DesktopFileParser::convertToJson(const QByteArray &key, ServiceTypeDefiniti
         kplugin[QStringLiteral("Category")] = value;
     } else if (key == QByteArrayLiteral("X-KDE-PluginInfo-License")) {
         kplugin[QStringLiteral("License")] = value;
+    } else if (key == QByteArrayLiteral("X-KDE-PluginInfo-Copyright")) {
+        kplugin[QStringLiteral("Copyright")] = value;
+    } else if (key.startsWith(QByteArrayLiteral("X-KDE-PluginInfo-Copyright["))) {
+        const QString languageSuffix = QString::fromUtf8(key.mid(qstrlen("X-KDE-PluginInfo-Copyright")));
+        kplugin[QStringLiteral("Copyright") + languageSuffix] = value;
     } else if (key == QByteArrayLiteral("X-KDE-PluginInfo-Version")) {
         kplugin[QStringLiteral("Version")] = value;
     } else if (key == QByteArrayLiteral("X-KDE-PluginInfo-Website")) {
@@ -500,6 +502,28 @@ void DesktopFileParser::convertToJson(const QByteArray &key, ServiceTypeDefiniti
         QJsonArray array;
         array.append(authorsObject);
         kplugin[QStringLiteral("Authors")] = array;
+    } else if (key == QByteArrayLiteral("X-KDE-PluginInfo-Authors")) {
+        const auto authors = deserializeList(value);
+        QJsonArray oldArray = kplugin.value(QStringLiteral("Authors")).toArray();
+        QJsonArray newArray;
+        for (int i = 0; i < authors.size(); ++i) {
+            QJsonObject authorsObject = oldArray.at(i).toObject();
+            // if the authors object doesn't exist yet this will create it
+            authorsObject[QStringLiteral("Name")] = authors[i];
+            newArray.append(authorsObject);
+        }
+        kplugin[QStringLiteral("Authors")] = newArray;
+    } else if (key == QByteArrayLiteral("X-KDE-PluginInfo-Emails")) {
+        const auto emails = deserializeList(value);
+        QJsonArray oldArray = kplugin.value(QStringLiteral("Authors")).toArray();
+        QJsonArray newArray;
+        for (int i = 0; i < emails.size(); ++i) {
+            QJsonObject authorsObject = oldArray.at(i).toObject();
+            // if the authors object doesn't exist yet this will create it
+            authorsObject[QStringLiteral("Email")] = emails[i];
+            newArray.append(authorsObject);
+        }
+        kplugin[QStringLiteral("Authors")] = newArray;
     } else if (key == QByteArrayLiteral("Name") || key.startsWith(QByteArrayLiteral("Name["))) {
         // TODO: also handle GenericName? does that make any sense, or is X-KDE-PluginInfo-Category enough?
         kplugin[QString::fromUtf8(key)] = value;
@@ -507,14 +531,21 @@ void DesktopFileParser::convertToJson(const QByteArray &key, ServiceTypeDefiniti
         kplugin[QStringLiteral("Description")] = value;
     } else if (key.startsWith(QByteArrayLiteral("Comment["))) {
         kplugin[QStringLiteral("Description") + QString::fromUtf8(key.mid(qstrlen("Comment")))] = value;
+    } else if (key == QByteArrayLiteral("InitialPreference")) {
+        kplugin[QStringLiteral("InitialPreference")] = value.toInt();
     } else if (key == QByteArrayLiteral("Hidden")) {
         DESKTOPTOJSON_VERBOSE_WARNING << "Hidden= key found in desktop file, this makes no sense"
                 " with metadata inside the plugin.";
         kplugin[QString::fromUtf8(key)] = (value.toLower() == QLatin1String("true"));
-    } else if (key == QByteArrayLiteral("Exec") || key == QByteArrayLiteral("Type")
-            || key == QByteArrayLiteral("X-KDE-Library") || key == QByteArrayLiteral("Encoding")) {
+    } else if (key == QByteArrayLiteral("Exec") ||
+               key == QByteArrayLiteral("Type") ||
+               key == QByteArrayLiteral("Actions") ||
+               key == QByteArrayLiteral("X-KDE-Library") ||
+               key == QByteArrayLiteral("Encoding")) {
         // Exec= doesn't make sense here, however some .desktop files (like e.g. in kdevelop) have a dummy value here
         // also the Type=Service entry is no longer needed
+        // Actions= is used as hack at least with the Dolphin KPart to report the different view mode options
+        // no plans yet to port that hack to JSON, so gently ignore it for now
         // X-KDE-Library is also not needed since we already have the library to read this metadata
         // Encoding= is also not converted as we always use utf-8 for reading
         DESKTOPTOJSON_VERBOSE_DEBUG << "Not converting key " << key << "=" << value;
@@ -528,7 +559,7 @@ bool DesktopFileParser::convert(const QString &src, const QStringList &serviceTy
 {
     QFile df(src);
     int lineNr = 0;
-    ServiceTypeDefinition serviceTypeDef = ServiceTypeDefinition::fromFiles(serviceTypes);
+    ServiceTypeDefinitions serviceTypeDef = ServiceTypeDefinitions::fromFiles(serviceTypes);
     readUntilDesktopEntryGroup(df, src, lineNr);
     DESKTOPTOJSON_VERBOSE_DEBUG << "Found [Desktop Entry] group in line" << lineNr;
     auto startPos = df.pos();
@@ -540,24 +571,27 @@ bool DesktopFileParser::convert(const QString &src, const QStringList &serviceTy
         if (!tokenizeKeyValue(df, src, key, value, lineNr)) {
             break;
         }
+        // some .desktop files still use the legacy ServiceTypes= key
         if (key == QByteArrayLiteral("X-KDE-ServiceTypes") || key == QByteArrayLiteral("ServiceTypes")) {
             const QString dotDesktop = QStringLiteral(".desktop");
             const QChar slashChar(QLatin1Char('/'));
             const auto serviceList = deserializeList(value);
 
-            for(const auto &service : serviceList) {
-                // some .desktop files still use the legacy ServiceTypes= key
-                QString absFileName = locateRelativeServiceType(
-                        service.toLower().replace(slashChar, QLatin1Char('-'))+dotDesktop);
-                if (absFileName.isEmpty()) {
-                    absFileName = locateRelativeServiceType(
-                        service.toLower().remove(slashChar)+dotDesktop);
+            for (const auto &service : serviceList) {
+                if (!serviceTypeDef.hasServiceType(service.toLatin1())) {
+                    // Make up the filename from the service type name. This assumes consistent naming...
+                    QString absFileName = locateRelativeServiceType(
+                            service.toLower().replace(slashChar, QLatin1Char('-')) + dotDesktop);
+                    if (absFileName.isEmpty()) {
+                        absFileName = locateRelativeServiceType(
+                                service.toLower().remove(slashChar) + dotDesktop);
+                    }
+                    if (absFileName.isEmpty()) {
+                        qCWarning(DESKTOPPARSER) << "Unable to find service type for service" << service << "listed in" << src;
+                    } else {
+                        serviceTypeDef.addFile(absFileName);
+                    }
                 }
-                if (absFileName.isEmpty()) {
-                    qCWarning(DESKTOPPARSER) << "Unable to find service type for service" << service;
-                }
-                else
-                    serviceTypeDef.addFile(absFileName);
             }
             break;
         }
