@@ -1,22 +1,10 @@
-/*  This file is part of the KDE project
-    Copyright (C) 2007 Matthias Kretz <kretz@kde.org>
-    Copyright (C) 2007 Bernhard Loos <nhuh.put@web.de>
+/*
+    This file is part of the KDE project
 
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Library General Public
-    License as published by the Free Software Foundation; either
-    version 2 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
-
-    You should have received a copy of the GNU Library General Public License
-    along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-    Boston, MA 02110-1301, USA.
+    SPDX-FileCopyrightText: 2007 Matthias Kretz <kretz@kde.org>
+    SPDX-FileCopyrightText: 2007 Bernhard Loos <nhuh.put@web.de>
 
+    SPDX-License-Identifier: LGPL-2.0-or-later
 */
 
 #include "kpluginfactory.h"
@@ -27,14 +15,9 @@
 
 Q_GLOBAL_STATIC(QObjectCleanupHandler, factorycleanup)
 
-extern int kLibraryDebugArea();
-
 KPluginFactory::KPluginFactory()
     : d_ptr(new KPluginFactoryPrivate)
 {
-    Q_D(KPluginFactory);
-    d->q_ptr = this;
-
     factorycleanup()->add(this);
 }
 
@@ -49,6 +32,20 @@ KPluginFactory::~KPluginFactory()
     delete d_ptr;
 }
 
+KPluginMetaData KPluginFactory::metaData() const
+{
+    Q_D(const KPluginFactory);
+
+    return d->metaData;
+}
+
+void KPluginFactory::setMetaData(const KPluginMetaData& metaData)
+{
+    Q_D(KPluginFactory);
+    d->metaData = metaData;
+}
+
+
 void KPluginFactory::registerPlugin(const QString &keyword, const QMetaObject *metaObject, CreateInstanceFunction instanceFunction)
 {
     Q_D(KPluginFactory);
@@ -89,6 +86,47 @@ void KPluginFactory::registerPlugin(const QString &keyword, const QMetaObject *m
     }
 }
 
+void KPluginFactory::registerPlugin(const QString &keyword, const QMetaObject *metaObject,
+                                    CreateInstanceWithMetaDataFunction instanceFunction)
+{
+    Q_D(KPluginFactory);
+
+    Q_ASSERT(metaObject);
+
+    // we allow different interfaces to be registered without keyword
+    if (!keyword.isEmpty()) {
+        if (d->createInstanceWithMetaDataHash.contains(keyword)) {
+            qCWarning(KCOREADDONS_DEBUG) << "A plugin with the keyword" << keyword << "was already registered. A keyword must be unique!";
+        }
+        d->createInstanceWithMetaDataHash.insert(keyword, {metaObject, instanceFunction});
+    } else {
+        const QList<KPluginFactoryPrivate::PluginWithMetadata> clashes(d->createInstanceWithMetaDataHash.values(keyword));
+        const QMetaObject *superClass = metaObject->superClass();
+        if (superClass) {
+            for (const KPluginFactoryPrivate::PluginWithMetadata &plugin : clashes) {
+                for (const QMetaObject *otherSuper = plugin.first->superClass(); otherSuper;
+                        otherSuper = otherSuper->superClass()) {
+                    if (superClass == otherSuper) {
+                        qCWarning(KCOREADDONS_DEBUG) << "Two plugins with the same interface(" << superClass->className() << ") were registered. Use keywords to identify the plugins.";
+                    }
+                }
+            }
+        }
+        for (const KPluginFactoryPrivate::PluginWithMetadata &plugin : clashes) {
+            superClass = plugin.first->superClass();
+            if (superClass) {
+                for (const QMetaObject *otherSuper = metaObject->superClass(); otherSuper;
+                        otherSuper = otherSuper->superClass()) {
+                    if (superClass == otherSuper) {
+                        qCWarning(KCOREADDONS_DEBUG) << "Two plugins with the same interface(" << superClass->className() << ") were registered. Use keywords to identify the plugins.";
+                    }
+                }
+            }
+        }
+        d->createInstanceWithMetaDataHash.insert(keyword, {metaObject, instanceFunction});
+    }
+}
+
 #if KCOREADDONS_BUILD_DEPRECATED_SINCE(4, 0)
 QObject *KPluginFactory::createObject(QObject *parent, const char *className, const QStringList &args)
 {
@@ -148,6 +186,24 @@ QObject *KPluginFactory::create(const char *iface, QWidget *parentWidget, QObjec
         }
     }
 
+    if (!obj) {
+        const QList<KPluginFactoryPrivate::PluginWithMetadata> candidates =
+            (d->createInstanceWithMetaDataHash.values(keyword));
+        // for !keyword.isEmpty() candidates.count() is 0 or 1
+
+        for (const KPluginFactoryPrivate::PluginWithMetadata &plugin : candidates) {
+            for (const QMetaObject *current = plugin.first; current; current = current->superClass()) {
+                if (0 == qstrcmp(iface, current->className())) {
+                    if (obj) {
+                        qCWarning(KCOREADDONS_DEBUG) << "ambiguous interface requested from a DSO containing more than one plugin";
+                    }
+                    obj = plugin.second(parentWidget, parent, d->metaData, args);
+                    break;
+                }
+            }
+        }
+    }
+
     if (obj) {
         emit objectCreated(obj);
     }
