@@ -1,19 +1,9 @@
-/*  This file is part of the KDE project
-    Copyright (C) 2007 Bernhard Loos <nhuh.put@web.de>
+/*
+    * This file is part of the KDE project
 
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Library General Public
-    License version 2 as published by the Free Software Foundation.
+    SPDX-FileCopyrightText: 2007 Bernhard Loos <nhuh.put@web.de>
 
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
-
-    You should have received a copy of the GNU Library General Public License
-    along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-    Boston, MA 02110-1301, USA.
+    SPDX-License-Identifier: LGPL-2.0-only
 */
 
 #include "kpluginloader.h"
@@ -38,20 +28,18 @@ class KPluginLoaderPrivate
     Q_DECLARE_PUBLIC(KPluginLoader)
 protected:
     KPluginLoaderPrivate(const QString &libname)
-        : name(libname),
-          loader(nullptr),
-          pluginVersion(~0U),
-          pluginVersionResolved(false)
+        : name(libname)
     {}
     ~KPluginLoaderPrivate()
     {}
 
-    KPluginLoader *q_ptr;
+    KPluginLoader *q_ptr = nullptr;
     const QString name;
     QString errorString;
-    QPluginLoader *loader;
-    quint32 pluginVersion;
-    bool pluginVersionResolved;
+    QPluginLoader *loader = nullptr;
+    quint32 pluginVersion = ~0U;
+    bool pluginVersionResolved = false;
+    bool isPluginMetaDataSet = false;
 };
 
 QString KPluginLoader::findPlugin(const QString &name)
@@ -121,6 +109,11 @@ KPluginFactory *KPluginLoader::factory()
         d->errorString = tr("The library %1 does not offer a KPluginFactory.").arg(d->name);
     }
 
+    if (!d->isPluginMetaDataSet && factory) {
+        factory->setMetaData(KPluginMetaData(*d->loader));
+        d->isPluginMetaDataSet = true;
+    }
+
     return factory;
 }
 
@@ -240,6 +233,9 @@ bool KPluginLoader::unload()
 void KPluginLoader::forEachPlugin(const QString &directory, std::function<void(const QString &)> callback)
 {
     QStringList dirsToCheck;
+#ifdef Q_OS_ANDROID
+    dirsToCheck << QCoreApplication::libraryPaths();
+#else
     if (QDir::isAbsolutePath(directory)) {
         dirsToCheck << directory;
     } else {
@@ -248,6 +244,7 @@ void KPluginLoader::forEachPlugin(const QString &directory, std::function<void(c
             dirsToCheck << libDir + QLatin1Char('/') + directory;
         }
     }
+#endif
 
     qCDebug(KCOREADDONS_DEBUG) << "Checking for plugins in" << dirsToCheck;
 
@@ -255,7 +252,15 @@ void KPluginLoader::forEachPlugin(const QString &directory, std::function<void(c
         QDirIterator it(dir, QDir::Files);
         while (it.hasNext()) {
             it.next();
+#ifdef Q_OS_ANDROID
+            QString prefix(QLatin1String("libplugins_") + QString(directory).replace(QLatin1Char('/'), QLatin1String("_")));
+            if (!prefix.endsWith(QLatin1Char('_'))) {
+                prefix.append(QLatin1Char('_'));
+            }
+            if (it.fileName().startsWith(prefix) && QLibrary::isLibrary(it.fileName())) {
+#else
             if (QLibrary::isLibrary(it.fileName())) {
+#endif
                 callback(it.fileInfo().absoluteFilePath());
             }
         }
