@@ -1,30 +1,21 @@
-/* This file is part of the KDE libraries
-   Copyright (C) 1999 Ian Zepp (icszepp@islc.net)
-   Copyright (C) 2006 by Dominic Battre <dominic@battre.de>
-   Copyright (C) 2006 by Martin Pool <mbp@canonical.com>
+/*
+    This file is part of the KDE libraries
 
-   This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
-   License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+    SPDX-FileCopyrightText: 1999 Ian Zepp <icszepp@islc.net>
+    SPDX-FileCopyrightText: 2006 Dominic Battre <dominic@battre.de>
+    SPDX-FileCopyrightText: 2006 Martin Pool <mbp@canonical.com>
 
-   This library is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
-
-   You should have received a copy of the GNU Library General Public License
-   along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301, USA.
+    SPDX-License-Identifier: LGPL-2.0-or-later
 */
 
 #include "kstringhandler.h"
 
 #include <stdlib.h>     // random()
 
+#if KCOREADDONS_BUILD_DEPRECATED_SINCE(5, 67)
 #include <QRegExp>            // for the word ranges
-#include <QCharRef>
+#endif
+#include <QRegularExpression>
 #include <QStringList>
 #include <QVector>
 
@@ -137,8 +128,14 @@ QStringList KStringHandler::perlSplit(const QChar &sep, const QString &s, int ma
     return l;
 }
 
+#if KCOREADDONS_BUILD_DEPRECATED_SINCE(5, 67)
 QStringList KStringHandler::perlSplit(const QRegExp &sep, const QString &s, int max)
 {
+    // nothing to split
+    if (s.isEmpty()) {
+        return QStringList();
+    }
+
     bool ignoreMax = 0 == max;
 
     QStringList l;
@@ -163,27 +160,49 @@ QStringList KStringHandler::perlSplit(const QRegExp &sep, const QString &s, int
 
     return l;
 }
+#endif
+
+QStringList KStringHandler::perlSplit(const QRegularExpression &sep, const QString &s, int max)
+{
+    // nothing to split
+    if (s.isEmpty()) {
+        return QStringList();
+    }
+
+    bool ignoreMax = max == 0;
+
+    QStringList list;
+
+    int start = 0;
+    QRegularExpressionMatchIterator iter = sep.globalMatch(s);
+    QRegularExpressionMatch match;
+    QString chunk;
+    while (iter.hasNext() && (ignoreMax || list.count() < max - 1)) {
+        match = iter.next();
+        chunk = s.mid(start, match.capturedStart() - start);
+        if (!chunk.isEmpty()) {
+            list.append(chunk);
+        }
+        start = match.capturedEnd();
+    }
+
+    // catch the remainder
+    chunk = s.mid(start, s.size() - start);
+    if (!chunk.isEmpty()) {
+        list.append(chunk);
+    }
+
+    return list;
+}
 
 QString KStringHandler::tagUrls(const QString &text)
 {
-    /*static*/ QRegExp urlEx(QStringLiteral("(www\\.(?!\\.)|(fish|(f|ht)tp(|s))://)[\\d\\w\\./,:_~\\?=&;#@\\-\\+\\%\\$\\(]+[\\d\\w/\\)]"));
-
     QString richText(text);
-    int urlPos = 0, urlLen;
-    while ((urlPos = urlEx.indexIn(richText, urlPos)) >= 0) {
-        urlLen = urlEx.matchedLength();
-        QString href = richText.mid(urlPos, urlLen);
-        // Qt doesn't support (?<=pattern) so we do it here
-        if ((urlPos > 0) && richText[urlPos - 1].isLetterOrNumber()) {
-            urlPos++;
-            continue;
-        }
-        // Don't use QString::arg since %01, %20, etc could be in the string
-        QString anchor = QLatin1String("<a href=\"") + href + QLatin1String("\">") + href + QLatin1String("</a>");
-        richText.replace(urlPos, urlLen, anchor);
-
-        urlPos += anchor.length();
-    }
+    const QRegularExpression urlEx(QStringLiteral("(www\\.(?!\\.)|(fish|ftp|http|https)://[\\d\\w\\./,:_~\\?=&;#@\\-\\+\\%\\$\\(\\)]+)"));
+    // the reference \1 is going to be replaced by the matched url
+    const QLatin1String regexBackRef(QLatin1String("\\1"));
+    const QString anchor = QLatin1String("<a href=\"") + regexBackRef + QLatin1String("\">") + regexBackRef + QLatin1String("</a>");
+    richText.replace(urlEx, anchor);
     return richText;
 }
 
