@@ -1,25 +1,13 @@
-/*  This file is part of the KDE Frameworks
+/*
+    This file is part of the KDE Frameworks
 
-    Copyright (C) 2013 Alex Merry <alex.merry@kdemail.net>
-    Copyright (C) 2013 John Layt <jlayt@kde.org>
-    Copyright (C) 2010 Michael Leupold <lemma@confuego.org>
-    Copyright (C) 2009 Michael Pyne <mpyne@kde.org>
-    Copyright (C) 2008 Albert Astals Cid <aacid@kde.org>
+    SPDX-FileCopyrightText: 2013 Alex Merry <alex.merry@kdemail.net>
+    SPDX-FileCopyrightText: 2013 John Layt <jlayt@kde.org>
+    SPDX-FileCopyrightText: 2010 Michael Leupold <lemma@confuego.org>
+    SPDX-FileCopyrightText: 2009 Michael Pyne <mpyne@kde.org>
+    SPDX-FileCopyrightText: 2008 Albert Astals Cid <aacid@kde.org>
 
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Library General Public
-    License as published by the Free Software Foundation; either
-    version 2 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
-
-    You should have received a copy of the GNU Library General Public License
-    along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-    Boston, MA 02110-1301, USA.
+    SPDX-License-Identifier: LGPL-2.0-or-later
 */
 
 #include "kformatprivate_p.h"
@@ -531,11 +519,22 @@ QString KFormatPrivate::formatRelativeDate(const QDate &date, QLocale::FormatTyp
 
 QString KFormatPrivate::formatRelativeDateTime(const QDateTime &dateTime, QLocale::FormatType format) const
 {
-    const qint64 daysTo = QDate::currentDate().daysTo(dateTime.date());
+    const QDateTime now = QDateTime::currentDateTime();
+    const qint64 daysTo = dateTime.daysTo(now);
     if (daysTo > 2 || daysTo < -2) {
         return m_locale.toString(dateTime, format);
     }
 
+    const auto secsToNow = dateTime.secsTo(now);
+    if (secsToNow >= 0 && secsToNow < 60 * 60) {
+        const int minutesToNow = secsToNow / 60;
+        if (minutesToNow <= 1) {
+            return tr("Just now");
+        } else {
+            return tr("%1 minutes ago").arg(minutesToNow);
+        }
+    }
+
     /*: relative datetime with %1 result of formatReleativeDate() and %2 the formatted time
         If this does not fit the grammar of your language please contact the i18n team to solve the problem */
     return tr("%1, %2").arg(formatRelativeDate(dateTime.date(), format), m_locale.toString(dateTime.time(), format));
