@@ -21,7 +21,6 @@ public:
     KListOpenFilesJobPrivate(KListOpenFilesJob *Job, const QDir &Path)
         : job(Job)
         , path(Path)
-        , hasEmittedResult(false)
     {
         QObject::connect(&lsofProcess, &QProcess::errorOccurred, [this](QProcess::ProcessError error) {
             lsofError(error);
@@ -56,7 +55,11 @@ private:
             return;
         }
         const QString out(QString::fromLocal8Bit(lsofProcess.readAll()));
+#if QT_VERSION < QT_VERSION_CHECK(5, 15, 0)
         const QVector<QStringRef> pidList = out.splitRef(QRegularExpression(QStringLiteral("\\s+")), QString::SkipEmptyParts);
+#else
+        const QVector<QStringRef> pidList = out.splitRef(QRegularExpression(QStringLiteral("\\s+")), Qt::SkipEmptyParts);
+#endif
         for (const auto &pidStr : pidList) {
             qint64 pid = pidStr.toLongLong();
             if (!pid) {
@@ -79,7 +82,7 @@ private:
 private:
     KListOpenFilesJob *job;
     const QDir path;
-    bool hasEmittedResult;
+    bool hasEmittedResult = false;
     QProcess lsofProcess;
     KProcessList::KProcessInfoList processInfoList;
 };
