@@ -1,22 +1,9 @@
 /*
     This file is part of the KDE libraries
 
-    Copyright (c) 2003,2007 Oswald Buddenhagen <ossi@kde.org>
+    SPDX-FileCopyrightText: 2003, 2007 Oswald Buddenhagen <ossi@kde.org>
 
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Library General Public
-    License as published by the Free Software Foundation; either
-    version 2 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
-
-    You should have received a copy of the GNU Library General Public License
-    along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-    Boston, MA 02110-1301, USA.
+    SPDX-License-Identifier: LGPL-2.0-or-later
 */
 
 #include "kshell.h"
@@ -211,7 +198,7 @@ QStringList KShell::splitArgs(const QString &args, Options flags, Errors *err)
                             if (pos >= args.length()) {
                                 goto quoteerr;
                             }
-                            cret += args.unicode()[pos++].toLatin1() & 31;
+                            cret += QChar::fromLatin1(args.unicode()[pos++].toLatin1() & 31);
                             break;
                         case 'x': {
                             if (pos >= args.length()) {
