@@ -1,30 +1,17 @@
 /*
- *  KUser - represent a user/account
- *  Copyright (C) 2002 Tim Jansen <tim@tjansen.de>
- *  Copyright (C) 2014 Alex Richardson <arichardson.kde@gmail.com>
- *
- *  This library is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU Library General Public
- *  License as published by the Free Software Foundation; either
- *  version 2 of the License, or (at your option) any later version.
- *
- *  This library is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- *  Library General Public License for more details.
- *
- *  You should have received a copy of the GNU Library General Public License
- *  along with this library; see the file COPYING.LIB.  If not, write to
- *  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- *  Boston, MA 02110-1301, USA.
- */
+    KUser - represent a user/account
+
+    SPDX-FileCopyrightText: 2002 Tim Jansen <tim@tjansen.de>
+    SPDX-FileCopyrightText: 2014 Alex Richardson <arichardson.kde@gmail.com>
+
+    SPDX-License-Identifier: LGPL-2.0-or-later
+*/
 
 #include "kuser.h"
 #include "config-getgrouplist.h"
 #include "config-accountsservice.h"
+#include "kcoreaddons_debug.h"
 
-#include <QMutableStringListIterator>
-#include <QDir>
 #include <QFileInfo>
 
 #include <pwd.h>
@@ -36,7 +23,7 @@
 #include <algorithm>  // std::find
 #include <functional> // std::function
 
-#if defined(__BIONIC__)
+#if defined(__BIONIC__) && __ANDROID_API__ < 26
 static inline struct passwd * getpwent() { return nullptr; }
 inline void setpwent() { }
 static inline void setgrent() { }
@@ -194,7 +181,7 @@ QString KUser::faceIconPath() const
         return pathToFaceIcon;
     }
 
-    pathToFaceIcon = QString(homeDir() + QLatin1Char('/') + QStringLiteral(".face.icon"));
+    pathToFaceIcon = homeDir() + QLatin1Char('/') + QLatin1String(".face.icon");
 
     if (QFileInfo(pathToFaceIcon).isReadable()) {
         return pathToFaceIcon;
@@ -222,7 +209,7 @@ static void listGroupsForUser(const char *name, gid_t gid, uint maxCount, Func h
     int result = getgrouplist(name, gid, gid_buffer.data(), &numGroups);
     if (result < 0 && uint(numGroups) < maxCount) {
         // getgrouplist returns -1 if the buffer was too small to store all entries, the required size is in numGroups
-        qDebug("Buffer was too small: %d, need %d", gid_buffer.size(), numGroups);
+        qCDebug(KCOREADDONS_DEBUG, "Buffer was too small: %d, need %d", gid_buffer.size(), numGroups);
         gid_buffer.resize(numGroups);
         numGroups = gid_buffer.size();
         getgrouplist(name, gid, gid_buffer.data(), &numGroups);
@@ -523,7 +510,7 @@ KUserId KUserId::fromName(const QString &name)
     QByteArray name8Bit = name.toLocal8Bit();
     struct passwd *p = ::getpwnam(name8Bit.constData());
     if (!p) {
-        qWarning("Failed to lookup user %s: %s", name8Bit.constData(), strerror(errno));
+        qCWarning(KCOREADDONS_DEBUG, "Failed to lookup user %s: %s", name8Bit.constData(), strerror(errno));
         return KUserId();
     }
     return KUserId(p->pw_uid);
@@ -537,7 +524,7 @@ KGroupId KGroupId::fromName(const QString &name)
     QByteArray name8Bit = name.toLocal8Bit();
     struct group *g = ::getgrnam(name8Bit.constData());
     if (!g) {
-        qWarning("Failed to lookup group %s: %s", name8Bit.constData(), strerror(errno));
+        qCWarning(KCOREADDONS_DEBUG, "Failed to lookup group %s: %s", name8Bit.constData(), strerror(errno));
         return KGroupId();
     }
     return KGroupId(g->gr_gid);
